@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 /*
-*	cvvs.c $Revision: 1.1.1.2 $ - PHP4 Interface to the RedHat CCVS API
+*	cvvs.c $Revision: 1.1.1.3 $ - PHP4 Interface to the RedHat CCVS API
 *	 -------
 *	 Interfaces RedHat's CCVS [Credit Card Verification System] <http://www.redhat.com/products/ccvs/>
 *	 This code is ported from an original php3 interface written by RedHat's Doug DeJulio <ddj@redhat.com>
@@ -27,10 +27,10 @@
 
 /*
 *	Code started on 2000.07.24@09.04.EST by Brendan W. McAdams <bmcadams@php.net>
-*	$Revision: 1.1.1.2 $
+*	$Revision: 1.1.1.3 $
 */
 
-static char const cvsid[] = "$Id: ccvs.c,v 1.1.1.2 2001/01/25 04:59:07 wsanchez Exp $";
+static char const cvsid[] = "$Id: ccvs.c,v 1.1.1.3 2001/07/19 00:18:58 zarzycki Exp $";
 
 #include <php.h>
 #include <stdlib.h>
@@ -193,6 +193,11 @@ PHP_FUNCTION(ccvs_new) /* cv_new() */
   }
 
   convert_to_string_ex(psess);
+  if (!Z_STRVAL_PP(psess)) {
+    php_error(E_WARNING, "Invalid session to ccvs_new()");
+    RETURN_FALSE;
+  }
+
   sess = hks_ptr_stringtoptr((*psess)->value.str.val);
 
   convert_to_string_ex(pinvoice);
