@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,15 +17,19 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: db.c,v 1.1.1.2 2001/01/25 04:59:10 wsanchez Exp $ */
+/* $Id: db.c,v 1.1.1.3 2001/07/19 00:19:02 zarzycki Exp $ */
 #define IS_EXT_MODULE
 
 #if 1
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "php.h"
 #include "php_globals.h"
 #include "safe_mode.h"
-#include "fopen-wrappers.h"
+#include "fopen_wrappers.h"
 #include "ext/standard/flock_compat.h" 
 #include "ext/standard/info.h"
 
@@ -120,7 +124,7 @@ static int php_dbm_key_exists(DBM *dbf, datum key_datum) {
 #define DBM_CREATE_MODE "a+b"
 #define DBM_NEW_MODE "w+b"
 #define DBM_DEFAULT_MODE "r"
-#define DBM_OPEN(filename, mode) V_FOPEN(filename, mode)
+#define DBM_OPEN(filename, mode) VCWD_FOPEN(filename, mode)
 #define DBM_CLOSE(dbf) fclose(dbf)
 #define DBM_STORE(dbf, key, value, mode) flatfile_store(dbf, key, value, mode)
 #define DBM_FETCH(dbf, key) flatfile_fetch(dbf, key)
@@ -322,7 +326,7 @@ dbm_info *php_dbm_open(char *filename, char *mode) {
 		strcat(lockfn, ".lck");
 
 #if NFS_HACK 
-		while((last_try = V_STAT(lockfn,&sb))==0) {
+		while((last_try = VCWD_STAT(lockfn,&sb))==0) {
 			retries++;
 			php_sleep(1);
 			if (retries>30) break;
@@ -336,7 +340,7 @@ dbm_info *php_dbm_open(char *filename, char *mode) {
 		}
 #else /* NFS_HACK */
 
-		lockfd = V_OPEN((lockfn,O_RDWR|O_CREAT,0644));
+		lockfd = VCWD_OPEN((lockfn,O_RDWR|O_CREAT,0644));
 
 		if (lockfd) {
 			flock(lockfd,LOCK_EX);
@@ -396,7 +400,7 @@ dbm_info *php_dbm_open(char *filename, char *mode) {
 
 #if NFS_HACK
 		if (lockfn) {
-			V_UNLINK(lockfn);
+			VCWD_UNLINK(lockfn);
 		}
 #endif
 		if (lockfn) efree(lockfn);
@@ -432,10 +436,10 @@ int php_dbm_close(zend_rsrc_list_entry *rsrc) {
 	dbf = info->dbf;
 
 #if NFS_HACK
-	V_UNLINK(info->lockfn);
+	VCWD_UNLINK(info->lockfn);
 #else
 	if (info->lockfn) {
-		lockfd = V_OPEN((info->lockfn,O_RDWR,0644));
+		lockfd = VCWD_OPEN((info->lockfn,O_RDWR,0644));
 		flock(lockfd,LOCK_UN);
 		close(lockfd);
 	}
