@@ -28,7 +28,11 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_ftp.c,v 1.1.1.3 2001/01/25 04:59:17 wsanchez Exp $ */
+/* $Id: php_ftp.c,v 1.1.1.4 2001/07/19 00:19:10 zarzycki Exp $ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "php.h"
 
@@ -592,9 +596,9 @@ PHP_FUNCTION(ftp_get)
 	}
 
 #ifdef PHP_WIN32
-	if ((outfp = V_FOPEN(Z_STRVAL_P(arg2), "wb")) == NULL) {
+	if ((outfp = VCWD_FOPEN(Z_STRVAL_P(arg2), "wb")) == NULL) {
 #else
-	if ((outfp = V_FOPEN(Z_STRVAL_P(arg2), "w")) == NULL) {
+	if ((outfp = VCWD_FOPEN(Z_STRVAL_P(arg2), "w")) == NULL) {
 #endif
 		fclose(tmpfp);
 		php_error(E_WARNING, "error opening %s", Z_STRVAL_P(arg2));
@@ -680,9 +684,9 @@ PHP_FUNCTION(ftp_put)
 	XTYPE(xtype, arg4);
 
 #ifdef PHP_WIN32
-	if ((infp = V_FOPEN(Z_STRVAL_P(arg3), "rb")) == NULL) {
+	if ((infp = VCWD_FOPEN(Z_STRVAL_P(arg3), "rb")) == NULL) {
 #else
-	if ((infp = V_FOPEN(Z_STRVAL_P(arg3), "r")) == NULL) {
+	if ((infp = VCWD_FOPEN(Z_STRVAL_P(arg3), "r")) == NULL) {
 #endif
 		php_error(E_WARNING, "error opening %s", Z_STRVAL_P(arg3));
 		RETURN_FALSE;
