@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -19,7 +19,11 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: ii.c,v 1.1.1.2 2001/01/25 04:59:23 wsanchez Exp $ */
+/* $Id: ii.c,v 1.1.1.3 2001/07/19 00:19:17 zarzycki Exp $ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "php.h"
 #include "php_globals.h"
@@ -139,7 +143,7 @@ static int _rollback_transaction(II_LINK *link)
   return 0;
 }
 
-static void close_ii_link(II_LINK *link)
+static void _close_ii_link(II_LINK *link)
 {
   IIAPI_DISCONNPARM disconnParm;
   IILS_FETCH();
@@ -166,7 +170,7 @@ static void close_ii_link(II_LINK *link)
 static void php_close_ii_link(zend_rsrc_list_entry *rsrc)
 {
 	II_LINK *link = (II_LINK *)rsrc->ptr;
-	close_ii_link(link);
+	_close_ii_link(link);
 }
 
 
@@ -177,7 +181,7 @@ static void _close_ii_plink(zend_rsrc_list_entry *rsrc)
 	II_LINK *link = (II_LINK *)rsrc->ptr;
 	IILS_FETCH();
 	
-	close_ii_link(link);
+	_close_ii_link(link);
 	IIG(num_persistent)--;
 }
 
@@ -248,12 +252,7 @@ static int php_ii_get_default_link(INTERNAL_FUNCTION_PARAMETERS IILS_DC)
 */
 PHP_MINIT_FUNCTION(ii)
 {
-  IIAPI_INITPARM initParm = {
-    -1,                /* timeout in ms, -1 = no timeout */
-    IIAPI_VERSION_1,   /* api version used */
-    0                  /* status (output) */
-  };
-  IILS_FETCH();
+  IIAPI_INITPARM initParm;
   
   REGISTER_INI_ENTRIES();
   
@@ -267,8 +266,10 @@ PHP_MINIT_FUNCTION(ii)
   REGISTER_LONG_CONSTANT("INGRES_NUM", II_NUM, CONST_CS | CONST_PERSISTENT);
   REGISTER_LONG_CONSTANT("INGRES_BOTH", II_BOTH, CONST_CS | CONST_PERSISTENT);
 
-
   /* Ingres api initialization */
+  initParm.in_timeout = -1;					/* timeout in ms, -1 = no timeout */
+  initParm.in_version = IIAPI_VERSION_1;	/* api version used */
+
   IIapi_initialize(&initParm);
   if (initParm.in_status==IIAPI_ST_SUCCESS) {
     return SUCCESS;
@@ -1065,10 +1066,10 @@ static void php_ii_fetch(INTERNAL_FUNCTION_PARAMETERS, II_LINK *ii_link, int res
 	  if(columnData[k-1].dv_null) { /* NULL value ? */
 
 	    if(result_type & II_NUM) {
-	      add_index_unset(return_value, i+k-1);
+	      add_index_null(return_value, i+k-1);
 	    }
 	    if(result_type & II_ASSOC) {
-	      add_assoc_unset(return_value, php_ii_field_name(ii_link, i+k-1));
+	      add_assoc_null(return_value, php_ii_field_name(ii_link, i+k-1));
 	    }
 
 	  } else { /* non NULL value */
