@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -14,12 +14,18 @@
    +----------------------------------------------------------------------+
    | Authors: Stig Sther Bakken <ssb@fast.no>                            |
    |          Andreas Karajannis <Andreas.Karajannis@gmd.de>              |
-   |          Frank M. Kromann <fmk@businessnet.dk> Support for DB/2 CLI  |
+   |          Frank M. Kromann <frank@frontbase.com> Support for DB/2 CLI |
+   |	      Kevin N. Shallow <kshallow@tampabay.rr.com> Velocis Support |
+   |		  Daniel R. Kalowsky <kalowsky@php.net>						  |
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_odbc.c,v 1.1.1.3 2001/01/25 04:59:45 wsanchez Exp $ */
+/* $Id: php_odbc.c,v 1.1.1.4 2001/07/19 00:19:41 zarzycki Exp $ */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+ 
 #include "php.h"
 #include "php_globals.h"
 
@@ -63,9 +69,11 @@ static int le_result, le_conn, le_pconn;
 
 #define SAFE_SQL_NTS(n) ((SWORD) ((n)?(SQL_NTS):0))
 
-static unsigned char a3_arg3_force_ref[] = { 3, BYREF_NONE, BYREF_ALLOW, BYREF_FORCE };
+static unsigned char a3_arg3_and_3_force_ref[] = { 3, BYREF_NONE, BYREF_FORCE, BYREF_FORCE };
 
 function_entry odbc_functions[] = {
+    PHP_FE(odbc_error, NULL)
+    PHP_FE(odbc_errormsg, NULL)
 	PHP_FE(odbc_setoption, NULL)
 	PHP_FE(odbc_autocommit, NULL)
 	PHP_FE(odbc_close, NULL)
@@ -82,13 +90,16 @@ function_entry odbc_functions[] = {
 	PHP_FE(odbc_prepare, NULL)
 	PHP_FE(odbc_execute, NULL)
 	PHP_FE(odbc_fetch_row, NULL)
-	PHP_FE(odbc_fetch_into, a3_arg3_force_ref)
+	PHP_FE(odbc_fetch_into, a3_arg3_and_3_force_ref)
 	PHP_FE(odbc_field_len, NULL)
 	PHP_FE(odbc_field_scale, NULL)
 	PHP_FE(odbc_field_name, NULL)
 	PHP_FE(odbc_field_type, NULL)
 	PHP_FE(odbc_field_num, NULL)
 	PHP_FE(odbc_free_result, NULL)
+#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30)
+	PHP_FE(odbc_next_result, NULL)
+#endif
 	PHP_FE(odbc_num_fields, NULL)
 	PHP_FE(odbc_num_rows, NULL)
 	PHP_FE(odbc_result, NULL)
@@ -100,14 +111,16 @@ function_entry odbc_functions[] = {
 	PHP_FE(odbc_columns, NULL)
 	PHP_FE(odbc_gettypeinfo, NULL)
 	PHP_FE(odbc_primarykeys, NULL)
-#if !defined(HAVE_DBMAKER) && !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) &&!defined(HAVE_SOLID_35)    /* not supported now */
+#if !defined(HAVE_DBMAKER) && !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) &&!defined(HAVE_SOLID_35) && !defined(HAVE_VELOCIS)    /* not supported now */
 	PHP_FE(odbc_columnprivileges, NULL)
 	PHP_FE(odbc_tableprivileges, NULL)
 #endif
-#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35)    /* not supported */
+#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35) /* not supported */
 	PHP_FE(odbc_foreignkeys, NULL)
 	PHP_FE(odbc_procedures, NULL)
+#if !defined(HAVE_VELOCIS)
 	PHP_FE(odbc_procedurecolumns, NULL)
+#endif
 #endif
 	PHP_FE(odbc_specialcolumns, NULL)
 	PHP_FE(odbc_statistics, NULL)
@@ -166,17 +179,29 @@ static void _free_odbc_result(zend_rsrc_list_entry *rsrc)
 	}
 }
 
+/*
+ * disconnect, and if it fails, then issue a rollback for any pending transaction (lurcher)
+ */
+
+static void safe_odbc_disconnect( void *handle )
+{
+	int ret;
+
+	ret = SQLDisconnect( handle );
+	if ( ret == SQL_ERROR )
+	{
+		SQLTransact( NULL, handle, SQL_ROLLBACK );
+		SQLDisconnect( handle );
+	}
+}
+
 static void _close_odbc_conn(zend_rsrc_list_entry *rsrc)
 {
 	odbc_connection *conn = (odbc_connection *)rsrc->ptr;
-	/* FIXME
-	 * Closing a connection will fail if there are
-	 * pending transactions. It is in the responsibility
-	 * of the user to avoid this.
-	 */
+
 	ODBCLS_FETCH();
 
-   	SQLDisconnect(conn->hdbc);
+   	safe_odbc_disconnect(conn->hdbc);
 	SQLFreeConnect(conn->hdbc);
 	SQLFreeEnv(conn->henv);
 	efree(conn);
@@ -188,7 +213,7 @@ static void _close_odbc_pconn(zend_rsrc_list_entry *rsrc)
 	odbc_connection *conn = (odbc_connection *)rsrc->ptr;
 	ODBCLS_FETCH();
 	
-	SQLDisconnect(conn->hdbc);
+	safe_odbc_disconnect(conn->hdbc);
 	SQLFreeConnect(conn->hdbc);
 	SQLFreeEnv(conn->henv);
 	free(conn);
@@ -236,6 +261,8 @@ static PHP_INI_DISP(display_defPW)
 #else
 		PUTS("********");
 #endif
+	} else {
+		PUTS("<i>no value</i>");
 	}
 }
 
@@ -319,7 +346,7 @@ PHP_MINIT_FUNCTION(odbc)
 {
 	ODBCLS_D;
 #ifdef SQLANY_BUG
-	HDBC    foobar;
+	ODBC_SQL_CONN_T foobar;
 	RETCODE rc;
 #endif
 
@@ -336,6 +363,7 @@ PHP_MINIT_FUNCTION(odbc)
 	le_pconn = zend_register_list_destructors_ex(NULL, _close_odbc_pconn, "odbc link persistent", module_number);
 	odbc_module_entry.type = type;
 	
+	REGISTER_STRING_CONSTANT("ODBC_TYPE", PHP_ODBC_TYPE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("ODBC_BINMODE_PASSTHRU", 0, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("ODBC_BINMODE_RETURN", 1, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("ODBC_BINMODE_CONVERT", 2, CONST_CS | CONST_PERSISTENT);
@@ -419,6 +447,8 @@ PHP_RINIT_FUNCTION(odbc)
 	
 	ODBCG(defConn) = -1;
 	ODBCG(num_links) = ODBCG(num_persistent);
+    memset(ODBCG(laststate), '\0', 6);
+    memset(ODBCG(lasterrormsg), '\0', SQL_MAX_MESSAGE_LENGTH);
 	return SUCCESS;
 }
 
@@ -465,22 +495,38 @@ void odbc_sql_error(ODBC_SQL_ERROR_PARAMS)
 	char	errormsg[SQL_MAX_MESSAGE_LENGTH];
 	SWORD	errormsgsize; /* Not used */
 	RETCODE rc;
+    ODBC_SQL_ENV_T henv;
+    ODBC_SQL_CONN_T conn;
 	ODBCLS_FETCH();
 
+    if (conn_resource) {
+        henv = conn_resource->henv;
+        conn = conn_resource->hdbc;
+    } else {
+        henv = SQL_NULL_HENV;
+        conn = SQL_NULL_HDBC;
+    }
+
 	/* This leads to an endless loop in many drivers! 
 	 *
 	   while(henv != SQL_NULL_HENV){
 		do {
 	 */
-			rc = SQLError(henv, conn, stmt, state,
-			    &error, errormsg, sizeof(errormsg)-1, &errormsgsize);
-	    	if (func) {
-		    	php_error(E_WARNING, "SQL error: %s, SQL state %s in %s",
-				   errormsg, state, func);
-			} else {
-		    	php_error(E_WARNING, "SQL error: %s, SQL state %s",
-						    errormsg, state);
-			}
+    rc = SQLError(henv, conn, stmt, state,
+                  &error, errormsg, sizeof(errormsg)-1, &errormsgsize);
+    if (conn_resource) {
+        memcpy(conn_resource->laststate, state, sizeof(state));
+        memcpy(conn_resource->lasterrormsg, errormsg, sizeof(errormsg));
+    }
+    memcpy(ODBCG(laststate), state, sizeof(state));
+    memcpy(ODBCG(lasterrormsg), errormsg, sizeof(errormsg));
+    if (func) {
+        php_error(E_WARNING, "SQL error: %s, SQL state %s in %s",
+                  errormsg, state, func);
+    } else {
+        php_error(E_WARNING, "SQL error: %s, SQL state %s",
+                  errormsg, state);
+    }
 	/*		
 		} while (SQL_SUCCEEDED(rc));
 	}
@@ -590,7 +636,7 @@ void odbc_transact(INTERNAL_FUNCTION_PARAMETERS, int type)
 	
 	rc = SQLTransact(conn->henv, conn->hdbc, (UWORD)((type)?SQL_COMMIT:SQL_ROLLBACK));
 	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLTransact");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLTransact");
 		RETURN_FALSE;
 	}
 
@@ -709,6 +755,7 @@ PHP_FUNCTION(odbc_longreadlen)
 }
 /* }}} */
 
+
 /* {{{ proto int odbc_prepare(int connection_id, string query)
    Prepares a statement for execution */
 PHP_FUNCTION(odbc_prepare)
@@ -747,7 +794,7 @@ PHP_FUNCTION(odbc_prepare)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -763,7 +810,7 @@ PHP_FUNCTION(odbc_prepare)
 			*/
 			if (SQLSetStmtOption(result->stmt, SQL_CURSOR_TYPE, SQL_CURSOR_DYNAMIC)
 				== SQL_ERROR) {
-				odbc_sql_error(conn->henv, conn->hdbc, result->stmt, " SQLSetStmtOption");
+				odbc_sql_error(conn, result->stmt, " SQLSetStmtOption");
 				SQLFreeStmt(result->stmt, SQL_DROP);
 				efree(result);
 				RETURN_FALSE;
@@ -774,11 +821,17 @@ PHP_FUNCTION(odbc_prepare)
 	}
 #endif
 
-	if ((rc = SQLPrepare(result->stmt, query, SQL_NTS)) != SQL_SUCCESS) {
-		odbc_sql_error(conn->henv, conn->hdbc, result->stmt, "SQLPrepare");
-		SQLFreeStmt(result->stmt, SQL_DROP);
-		RETURN_FALSE;
-	}
+    rc = SQLPrepare(result->stmt, query, SQL_NTS);
+    switch (rc) {
+    case SQL_SUCCESS:
+        break;
+    case SQL_SUCCESS_WITH_INFO:
+        odbc_sql_error(conn, result->stmt, "SQLPrepare");
+        break;
+    default:
+        odbc_sql_error(conn, result->stmt, "SQLPrepare");
+        RETURN_FALSE;
+    }
 	
 	SQLNumParams(result->stmt, &(result->numparams));
     SQLNumResultCols(result->stmt, &(result->numcols));
@@ -916,7 +969,7 @@ PHP_FUNCTION(odbc_execute)
 	rc = SQLFreeStmt(result->stmt, SQL_CLOSE);
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc, result->stmt, "SQLFreeStmt");	
+		odbc_sql_error(result->conn_ptr, result->stmt, "SQLFreeStmt");	
 	}
 
 	rc = SQLExecute(result->stmt);
@@ -933,10 +986,16 @@ PHP_FUNCTION(odbc_execute)
 			}
 		}
 	} else {
-		if (rc != SQL_SUCCESS) {
-			odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc, result->stmt, "SQLExecute");
-			RETVAL_FALSE;
-		}
+        switch (rc) {
+        case SQL_SUCCESS:
+            break;
+        case SQL_NO_DATA_FOUND:
+        case SQL_SUCCESS_WITH_INFO:
+            odbc_sql_error(result->conn_ptr, result->stmt, "SQLExecute");
+            break;
+        default:
+            RETVAL_FALSE;
+        }
 	}	
 	
 	if (result->numparams > 0) {
@@ -948,7 +1007,8 @@ PHP_FUNCTION(odbc_execute)
 		efree(params);
 	}
 
-	if (rc == SQL_SUCCESS) {
+	if (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO 
+        || rc == SQL_NO_DATA_FOUND) {
 		RETVAL_TRUE;
 	}
 
@@ -986,7 +1046,7 @@ PHP_FUNCTION(odbc_cursor)
 
 	rc = SQLGetInfo(result->conn_ptr->hdbc,SQL_MAX_CURSOR_NAME_LEN,
 					(void *)&max_len,0,&len);
-	if (rc != SQL_SUCCESS) {
+	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
 		RETURN_FALSE;
 	}
 	
@@ -997,7 +1057,7 @@ PHP_FUNCTION(odbc_cursor)
 			RETURN_FALSE;
 		}
 		rc = SQLGetCursorName(result->stmt,cursorname,(SWORD)max_len,&len);
-		if (rc != SQL_SUCCESS) {
+		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
 			char    state[6];     /* Not used */
 	 		SDWORD  error;        /* Not used */
 			char    errormsg[255];
@@ -1010,8 +1070,7 @@ PHP_FUNCTION(odbc_cursor)
 			if (!strncmp(state,"S1015",5)) {
 				sprintf(cursorname,"php_curs_%d", (int)result->stmt);
 				if (SQLSetCursorName(result->stmt,cursorname,SQL_NTS) != SQL_SUCCESS) {
-					odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc,
-									result->stmt, "SQLSetCursorName");
+					odbc_sql_error(result->conn_ptr, result->stmt, "SQLSetCursorName");
 					RETVAL_FALSE;
 				} else {
 					RETVAL_STRING(cursorname,1);
@@ -1074,7 +1133,7 @@ PHP_FUNCTION(odbc_exec)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -1090,7 +1149,7 @@ PHP_FUNCTION(odbc_exec)
 			 */
 			if (SQLSetStmtOption(result->stmt, SQL_CURSOR_TYPE, SQL_CURSOR_DYNAMIC)
 				== SQL_ERROR) {
-				odbc_sql_error(conn->henv, conn->hdbc, result->stmt, " SQLSetStmtOption");
+				odbc_sql_error(conn, result->stmt, " SQLSetStmtOption");
 				SQLFreeStmt(result->stmt, SQL_DROP);
 				efree(result);
 				RETURN_FALSE;
@@ -1102,11 +1161,13 @@ PHP_FUNCTION(odbc_exec)
 #endif
 
 	rc = SQLExecDirect(result->stmt, query, SQL_NTS);
-	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
+	if (rc != SQL_SUCCESS 
+        && rc != SQL_SUCCESS_WITH_INFO
+        && rc != SQL_NO_DATA_FOUND) { 
 		/* XXX FIXME we should really check out SQLSTATE with SQLError
 		 * in case rc is SQL_SUCCESS_WITH_INFO here.
 		 */
-		odbc_sql_error(conn->henv, conn->hdbc, result->stmt, "SQLExecDirect"); 
+		odbc_sql_error(conn, result->stmt, "SQLExecDirect"); 
 		SQLFreeStmt(result->stmt, SQL_DROP);
 		efree(result);
 		RETURN_FALSE;
@@ -1231,7 +1292,7 @@ static void php_odbc_fetch_hash(INTERNAL_FUNCTION_PARAMETERS, int result_type)
         buf, result->longreadlen + 1, &result->values[i].vallen);
 
      if (rc == SQL_ERROR) {
-     odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc, result->stmt, "SQLGetData");
+     odbc_sql_error(result->conn_ptr, result->stmt, "SQLGetData");
      efree(buf);
      RETURN_FALSE;
     }
@@ -1312,6 +1373,7 @@ PHP_FUNCTION(odbc_fetch_into)
 		case 3:
 			if (zend_get_parameters_ex(3, &pv_res, &pv_row, &pv_res_arr) == FAILURE)
 				WRONG_PARAM_COUNT;
+			SEPARATE_ZVAL(pv_row);
 			convert_to_long_ex(pv_row);
 			rownum = (*pv_row)->value.lval;
 			break;
@@ -1328,11 +1390,6 @@ PHP_FUNCTION(odbc_fetch_into)
 		WRONG_PARAM_COUNT;
 	}
 #endif
-	
-	if (!ParameterPassedByReference(ht, numArgs)) {
-		php_error(E_WARNING, "Array not passed by reference in call to odbc_fetch_into()");
-		RETURN_FALSE;
-	}
 
 	ZEND_FETCH_RESOURCE(result, odbc_result *, pv_res, -1, "ODBC result", le_result);
 	
@@ -1396,7 +1453,7 @@ PHP_FUNCTION(odbc_fetch_into)
 								buf, result->longreadlen + 1, &result->values[i].vallen);
 
 					if (rc == SQL_ERROR) {
-					odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc, result->stmt, "SQLGetData");
+					odbc_sql_error(result->conn_ptr, result->stmt, "SQLGetData");
 					efree(buf);
 					RETURN_FALSE;
 				}
@@ -1616,7 +1673,7 @@ PHP_FUNCTION(odbc_result)
                             field, fieldsize, &result->values[field_ind].vallen);
             
             if (rc == SQL_ERROR) {
-                odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc, result->stmt, "SQLGetData");
+                odbc_sql_error(result->conn_ptr, result->stmt, "SQLGetData");
                 efree(field);
                 RETURN_FALSE;
             }
@@ -1659,7 +1716,7 @@ PHP_FUNCTION(odbc_result)
                             field, fieldsize, &result->values[field_ind].vallen);
 
 		if (rc == SQL_ERROR) {
-			odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc, result->stmt, "SQLGetData");
+			odbc_sql_error(result->conn_ptr, result->stmt, "SQLGetData");
             efree(field);
 			RETURN_FALSE;
 		}
@@ -1765,7 +1822,7 @@ PHP_FUNCTION(odbc_result_all)
 					php_printf("<td>");
 
 					if (rc == SQL_ERROR) {
-						odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc, result->stmt, "SQLGetData");
+						odbc_sql_error(result->conn_ptr, result->stmt, "SQLGetData");
 						php_printf("</td></tr></table>");
 						efree(buf);
 						RETURN_FALSE;
@@ -1864,28 +1921,33 @@ int odbc_sqlconnect(odbc_connection **conn, char *db, char *uid, char *pwd, int
 	if (cur_opt != SQL_CUR_DEFAULT) {
 		rc = SQLSetConnectOption((*conn)->hdbc, SQL_ODBC_CURSORS, cur_opt);
 		if (rc != SQL_SUCCESS) {  /* && rc != SQL_SUCCESS_WITH_INFO ? */
-			odbc_sql_error((*conn)->henv, (*conn)->hdbc, SQL_NULL_HSTMT, "SQLSetConnectOption");
+			odbc_sql_error(*conn, SQL_NULL_HSTMT, "SQLSetConnectOption");
 			SQLFreeConnect((*conn)->hdbc);
 			pefree(*conn, persistent);
 			return FALSE;
 		}
 	}
-#ifdef HAVE_EMPRESS
+/*  Possible fix for bug #10250
+ *  Needs testing on UnixODBC < 2.0.5 though. */
+ #if defined(HAVE_EMPRESS) || defined(HAVE_UNIXODBC)
+/* *  Uncomment the line above, and comment line below to fully test 
+ * #ifdef HAVE_EMPRESS */
 	{
 		int     direct = 0;
 		char    dsnbuf[300];
 		short   dsnbuflen;
 		char    *ldb = 0;
+		int		ldb_len = 0;
 
 		if (strstr((char*)db, ";")) {
 			direct = 1;
-			if (uid && !strstr ((char*)db, "uid") &&
-						!strstr((char*)db, "UID")) {
-				ldb = (char*)emalloc(strlen(db) + strlen(uid) + strlen(pwd) + 12);
+			if (uid && !strstr ((char*)db, "uid") && !strstr((char*)db, "UID")) {
+				ldb = (char*) emalloc(strlen(db) + strlen(uid) + strlen(pwd) + 12);
 				sprintf(ldb, "%s;UID=%s;PWD=%s", db, uid, pwd);
 			} else {
-				ldb = (char*)emalloc(strlen(db) + 1);
-				strcat(ldb, db);
+				ldb_len = strlen(db)+1;
+				ldb = (char*) emalloc(ldb_len);
+				memcpy(ldb, db, ldb_len);
 			}
 		}
 
@@ -1902,7 +1964,7 @@ int odbc_sqlconnect(odbc_connection **conn, char *db, char *uid, char *pwd, int
 #endif
 #endif
 	if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
-		odbc_sql_error((*conn)->henv, (*conn)->hdbc, SQL_NULL_HSTMT, "SQLConnect");
+		odbc_sql_error(*conn, SQL_NULL_HSTMT, "SQLConnect");
 		SQLFreeConnect((*conn)->hdbc);
 		pefree((*conn), persistent);
 		return FALSE;
@@ -2055,7 +2117,7 @@ try_and_get_another_connection:
 
 				if(ret != SQL_SUCCESS){
 					zend_hash_del(&EG(persistent_list), hashed_details, hashed_len + 1);
-					SQLDisconnect(db_conn->hdbc);
+					safe_odbc_disconnect(db_conn->hdbc);
 					SQLFreeConnect(db_conn->hdbc);
 					goto try_and_get_another_connection;
 				}
@@ -2120,15 +2182,15 @@ PHP_FUNCTION(odbc_close)
 	int i;
 	int type;
 	int is_pconn = 0;
+	int found_resource_type = le_conn;
 	ODBCLS_FETCH();
 
     if (zend_get_parameters_ex(1, &pv_conn) == FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
 
-	conn = (odbc_connection *) zend_fetch_resource(pv_conn, -1, "ODBC-Link", NULL, 1, le_conn);
-	if(!conn){
-		ZEND_FETCH_RESOURCE(conn, odbc_connection *, pv_conn, -1, "ODBC-Link", le_pconn);
+	conn = (odbc_connection *) zend_fetch_resource(pv_conn, -1, "ODBC-Link", &found_resource_type, 2, le_conn, le_pconn);
+	if (found_resource_type==le_pconn) {
 		is_pconn = 1;
 	}
 
@@ -2171,6 +2233,56 @@ PHP_FUNCTION(odbc_num_rows)
 }
 /* }}} */
 
+#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30)
+/* {{{ proto bool next_result(int result_id)
+   Checks if multiple results are avaiable */
+PHP_FUNCTION(odbc_next_result)
+{
+	odbc_result   *result;
+	pval     **pv_res;
+	int rc, i;
+
+ 	if (zend_get_parameters_ex(1, &pv_res) == FAILURE) {
+		WRONG_PARAM_COUNT;
+	}                            
+	ZEND_FETCH_RESOURCE(result, odbc_result *, pv_res, -1, "ODBC result", le_result); 
+
+	if (result->values) {
+		for(i = 0; i < result->numcols; i++) {
+			if (result->values[i].value)
+				efree(result->values[i].value);
+		}
+		efree(result->values);
+		result->values = NULL;
+	}
+
+	result->fetched = 0;
+	rc = SQLMoreResults(result->stmt);
+	if (rc == SQL_SUCCESS) {
+		RETURN_TRUE;
+	}
+	else if (rc == SQL_SUCCESS_WITH_INFO) {
+		rc = SQLFreeStmt(result->stmt, SQL_UNBIND);
+		SQLNumParams(result->stmt, &(result->numparams));
+		SQLNumResultCols(result->stmt, &(result->numcols));
+
+		if (result->numcols > 0) {
+			if (!odbc_bindcols(result)) {
+				efree(result);
+				RETVAL_FALSE;
+			}
+		} else {
+			result->values = NULL;
+		}
+		RETURN_TRUE;
+	}
+	else {
+		RETURN_FALSE;
+	}
+}
+/* }}} */
+#endif
+
 /* {{{ proto int odbc_num_fields(int result_id)
    Get number of columns in a result */
 PHP_FUNCTION(odbc_num_fields)
@@ -2335,17 +2447,13 @@ PHP_FUNCTION(odbc_autocommit)
 
 	ZEND_FETCH_RESOURCE2(conn, odbc_connection *, pv_conn, -1, "ODBC-Link", le_conn, le_pconn);
 	
-#ifndef HAVE_DBMAKER	
-	if ((*pv_onoff)) {
-#else
 	if (pv_onoff && (*pv_onoff)) {
-#endif
 		convert_to_long_ex(pv_onoff);
 		rc = SQLSetConnectOption(conn->hdbc, SQL_AUTOCOMMIT,
 								 ((*pv_onoff)->value.lval) ?
 								 SQL_AUTOCOMMIT_ON : SQL_AUTOCOMMIT_OFF);
 		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
-			odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "Set autocommit");
+			odbc_sql_error(conn, SQL_NULL_HSTMT, "Set autocommit");
 			RETURN_FALSE;
 		}
 		RETVAL_TRUE;
@@ -2354,7 +2462,7 @@ PHP_FUNCTION(odbc_autocommit)
 
 		rc = SQLGetConnectOption(conn->hdbc, SQL_AUTOCOMMIT, (PTR)&status);
 		if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
-			odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "Get commit status");
+			odbc_sql_error(conn, SQL_NULL_HSTMT, "Get commit status");
 			RETURN_FALSE;
 		}
 		RETVAL_LONG((long)status);
@@ -2378,6 +2486,59 @@ PHP_FUNCTION(odbc_rollback)
 }
 /* }}} */
 
+static void php_odbc_lasterror(INTERNAL_FUNCTION_PARAMETERS, int mode)
+{
+	odbc_connection *conn;
+	pval **pv_handle;
+    char *ptr;
+    int argc, len;
+
+    argc = ZEND_NUM_ARGS();
+
+    if (argc > 1 || zend_get_parameters_ex(argc, &pv_handle)) {
+		WRONG_PARAM_COUNT;
+	}                            
+ 
+    if (mode == 0) {  /* last state */
+        len = 6;
+    } else { /* last error message */
+        len = SQL_MAX_MESSAGE_LENGTH;
+    }
+    ptr = ecalloc(len + 1, 1);
+    if (argc == 1) {
+        ZEND_FETCH_RESOURCE2(conn, odbc_connection *, pv_handle, -1, "ODBC-Link", le_conn, le_pconn);
+        if (mode == 0) {
+            strlcpy(ptr, conn->laststate, len+1);
+        } else {
+            strlcpy(ptr, conn->lasterrormsg, len+1);
+        }
+    } else {
+		ODBCLS_FETCH();
+
+        if (mode == 0) {
+            strlcpy(ptr, ODBCG(laststate), len+1);
+        } else {
+            strlcpy(ptr, ODBCG(lasterrormsg), len+1);
+        }
+    }
+    RETVAL_STRING(ptr, 0);
+}
+
+/* {{{ proto string odbc_error([int connection_id])
+   Get the last error code */
+PHP_FUNCTION(odbc_error)
+{
+    php_odbc_lasterror(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
+}
+/* }}}    */
+
+/* {{{ proto string odbc_errormsg([int connection_id])
+   Get the last error message */
+PHP_FUNCTION(odbc_errormsg)
+{
+    php_odbc_lasterror(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
+}
+/* }}}    */
 
 /* {{{ proto int odbc_setoption(int conn_id|result_id, int which, int option, int value)
    Sets connection or statement options */
@@ -2411,7 +2572,7 @@ PHP_FUNCTION(odbc_setoption)
 			}
 			rc = SQLSetConnectOption(conn->hdbc, (unsigned short)((*pv_opt)->value.lval), (*pv_val)->value.lval);
 			if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
-				odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SetConnectOption");
+				odbc_sql_error(conn, SQL_NULL_HSTMT, "SetConnectOption");
 				RETURN_FALSE;
 			}
 			break;
@@ -2421,7 +2582,7 @@ PHP_FUNCTION(odbc_setoption)
 			rc = SQLSetStmtOption(result->stmt, (unsigned short)((*pv_opt)->value.lval), ((*pv_val)->value.lval));
 
 			if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) {
-				odbc_sql_error(result->conn_ptr->henv, result->conn_ptr->hdbc, result->stmt, "SetStmtOption");
+				odbc_sql_error(result->conn_ptr, result->stmt, "SetStmtOption");
 				RETURN_FALSE;
 			}
 			break;
@@ -2485,11 +2646,14 @@ PHP_FUNCTION(odbc_tables)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
 
+	/* This hack is needed to access table information in Access databases (fmk) */
+	if (table && strlen(table) && schema && !strlen(schema)) schema = NULL;
+
 	rc = SQLTables(result->stmt, 
             cat, SAFE_SQL_NTS(cat), 
             schema, SAFE_SQL_NTS(schema),
@@ -2497,7 +2661,7 @@ PHP_FUNCTION(odbc_tables)
             type, SAFE_SQL_NTS(type));
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLTables");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLTables");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2565,7 +2729,7 @@ PHP_FUNCTION(odbc_columns)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2577,7 +2741,7 @@ PHP_FUNCTION(odbc_columns)
             column, SAFE_SQL_NTS(column));
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLColumns");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLColumns");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2599,7 +2763,7 @@ PHP_FUNCTION(odbc_columns)
 }
 /* }}} */
 
-#if !defined(HAVE_DBMAKER) && !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35)
+#if !defined(HAVE_DBMAKER) && !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35) && !defined(HAVE_VELOCIS)
 /* {{{ proto int odbc_columnprivileges(int connection_id, string catalog, string schema, string table, string column)
    Returns a result identifier that can be used to fetch a list of columns and associated privileges for the specified table */
 PHP_FUNCTION(odbc_columnprivileges)
@@ -2644,7 +2808,7 @@ PHP_FUNCTION(odbc_columnprivileges)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2656,7 +2820,7 @@ PHP_FUNCTION(odbc_columnprivileges)
             column, SAFE_SQL_NTS(column));
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLColumnPrivileges");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLColumnPrivileges");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2742,7 +2906,7 @@ PHP_FUNCTION(odbc_foreignkeys)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2756,7 +2920,7 @@ PHP_FUNCTION(odbc_foreignkeys)
             ftable, SAFE_SQL_NTS(ftable) );
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLForeignKeys");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLForeignKeys");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2821,7 +2985,7 @@ PHP_FUNCTION(odbc_gettypeinfo)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2829,7 +2993,7 @@ PHP_FUNCTION(odbc_gettypeinfo)
 	rc = SQLGetTypeInfo(result->stmt, data_type );
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLGetTypeInfo");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLGetTypeInfo");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2893,7 +3057,7 @@ PHP_FUNCTION(odbc_primarykeys)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2904,7 +3068,7 @@ PHP_FUNCTION(odbc_primarykeys)
             table, SAFE_SQL_NTS(table) );
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLPrimaryKeys");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLPrimaryKeys");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2926,7 +3090,7 @@ PHP_FUNCTION(odbc_primarykeys)
 }
 /* }}} */
 
-#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35)
+#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35) && !defined(HAVE_VELOCIS)
 /* {{{ proto int odbc_procedurecolumns(int connection_id [, string qualifier, string owner, string proc, string column])
    Returns a result identifier containing the list of input and output parameters, as well as the columns that make up the result set for the specified procedures */
 PHP_FUNCTION(odbc_procedurecolumns)
@@ -2975,7 +3139,7 @@ PHP_FUNCTION(odbc_procedurecolumns)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -2987,7 +3151,7 @@ PHP_FUNCTION(odbc_procedurecolumns)
             col, SAFE_SQL_NTS(col) );
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLProcedureColumns");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLProcedureColumns");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -3057,7 +3221,7 @@ PHP_FUNCTION(odbc_procedures)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -3068,7 +3232,7 @@ PHP_FUNCTION(odbc_procedures)
             proc, SAFE_SQL_NTS(proc) );
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLProcedures");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLProcedures");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -3143,7 +3307,7 @@ PHP_FUNCTION(odbc_specialcolumns)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -3157,7 +3321,7 @@ PHP_FUNCTION(odbc_specialcolumns)
             nullable);
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLSpecialColumns");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLSpecialColumns");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -3228,7 +3392,7 @@ PHP_FUNCTION(odbc_statistics)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -3241,7 +3405,7 @@ PHP_FUNCTION(odbc_statistics)
             reserved);
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLStatistics");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLStatistics");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -3263,7 +3427,7 @@ PHP_FUNCTION(odbc_statistics)
 }
 /* }}} */
 
-#if !defined(HAVE_DBMAKER) && !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35)
+#if !defined(HAVE_DBMAKER) && !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35) && !defined(HAVE_VELOCIS)
 /* {{{ proto int odbc_tableprivileges(int connection_id, string qualifier, string owner, string name)
    Returns a result identifier containing a list of tables and the privileges associated with each table */
 PHP_FUNCTION(odbc_tableprivileges)
@@ -3306,7 +3470,7 @@ PHP_FUNCTION(odbc_tableprivileges)
 	}
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLAllocStmt");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLAllocStmt");
 		efree(result);
 		RETURN_FALSE;
 	}
@@ -3317,7 +3481,7 @@ PHP_FUNCTION(odbc_tableprivileges)
             table, SAFE_SQL_NTS(table));
 
 	if (rc == SQL_ERROR) {
-		odbc_sql_error(conn->henv, conn->hdbc, SQL_NULL_HSTMT, "SQLTablePrivileges");
+		odbc_sql_error(conn, SQL_NULL_HSTMT, "SQLTablePrivileges");
 		efree(result);
 		RETURN_FALSE;
 	}
