@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -13,16 +13,20 @@
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
    | Authors: Zeev Suraski <zeev@zend.com>                                |
-   |          Jouni Ahto <jah@mork.net> (large object interface)          |
+   |          Jouni Ahto <jouni.ahto@exdec.fi> (large object interface)   |
    +----------------------------------------------------------------------+
  */
  
-/* $Id: pgsql.c,v 1.1.1.3 2001/01/25 04:59:53 wsanchez Exp $ */
+/* $Id: pgsql.c,v 1.1.1.4 2001/07/19 00:19:52 zarzycki Exp $ */
 
 #include <stdlib.h>
 
 #define PHP_PGSQL_PRIVATE 1
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "php.h"
 #include "php_ini.h"
 #include "ext/standard/php_standard.h"
@@ -31,6 +35,9 @@
 
 #if HAVE_PGSQL
 
+#ifndef InvalidOid
+#define InvalidOid ((Oid) 0)
+#endif
 
 #define PGSQL_ASSOC		1<<0
 #define PGSQL_NUM		1<<1
@@ -43,6 +50,7 @@ function_entry pgsql_functions[] = {
 	PHP_FE(pg_pconnect,		NULL)
 	PHP_FE(pg_close,		NULL)
 	PHP_FE(pg_cmdtuples,	NULL)
+	PHP_FE(pg_last_notice,  NULL)
 	PHP_FE(pg_dbname,		NULL)
 	PHP_FE(pg_errormessage,	NULL)
 	PHP_FE(pg_trace,		NULL)
@@ -93,7 +101,7 @@ zend_module_entry pgsql_module_entry = {
 	PHP_MINIT(pgsql),
 	PHP_MSHUTDOWN(pgsql),
 	PHP_RINIT(pgsql),
-	NULL,
+	PHP_RSHUTDOWN(pgsql),
 	PHP_MINFO(pgsql),
 	STANDARD_MODULE_PROPERTIES
 };
@@ -114,13 +122,13 @@ static void php_pgsql_set_default_link(int id)
 {   
 	PGLS_FETCH();
 
-    if (PGG(default_link)!=-1) {
-        zend_list_delete(PGG(default_link));
-    }
-    if (PGG(default_link) != id) {
-        PGG(default_link) = id;
-        zend_list_addref(id);
-    }
+	zend_list_addref(id);
+
+	if (PGG(default_link) != -1) {
+		zend_list_delete(PGG(default_link));
+	}
+
+	PGG(default_link) = id;
 }
 
 
@@ -142,6 +150,41 @@ static void _close_pgsql_plink(zend_rsrc_list_entry *rsrc)
 	PQfinish(link);
 	PGG(num_persistent)--;
 	PGG(num_links)--;
+	if(PGG(last_notice) != NULL) {
+		efree(PGG(last_notice));
+	}
+}
+
+
+static void
+_notice_handler(void *arg, const char *message)
+{
+	PGLS_FETCH();
+
+	if (! PGG(ignore_notices)) {
+		php_log_err((char *) message);
+		if (PGG(last_notice) != NULL) {
+			efree(PGG(last_notice));
+		}
+		PGG(last_notice) = estrdup(message);
+	}
+}
+
+
+static int _rollback_transactions(zend_rsrc_list_entry *rsrc)
+{
+	PGconn *link;
+	PGLS_FETCH();
+	
+	if (rsrc->type != le_plink) return 0;
+
+	link = (PGconn *) rsrc->ptr;
+
+	PGG(ignore_notices) = 1;
+	PQexec(link,"BEGIN;ROLLBACK;");
+	PGG(ignore_notices) = 0;
+
+	return 0;
 }
 
 
@@ -169,6 +212,8 @@ PHP_INI_END()
 static void php_pgsql_init_globals(PGLS_D)
 {
 	PGG(num_persistent) = 0;
+	PGG(ignore_notices) = 0;
+	PGG(last_notice) = NULL;
 }
 
 PHP_MINIT_FUNCTION(pgsql)
@@ -212,6 +257,14 @@ PHP_RINIT_FUNCTION(pgsql)
 	return SUCCESS;
 }
 
+PHP_RSHUTDOWN_FUNCTION(pgsql)
+{
+	ELS_FETCH();
+	zend_hash_apply(&EG(persistent_list), (apply_func_t) _rollback_transactions);
+	return SUCCESS;
+}
+
+
 
 PHP_MINFO_FUNCTION(pgsql)
 {
@@ -229,6 +282,8 @@ PHP_MINFO_FUNCTION(pgsql)
 	DISPLAY_INI_ENTRIES();
 
 }
+
+
 void php_pgsql_do_connect(INTERNAL_FUNCTION_PARAMETERS,int persistent)
 {
 	char *host=NULL,*port=NULL,*options=NULL,*tty=NULL,*dbname=NULL,*connstring=NULL;
@@ -339,11 +394,16 @@ void php_pgsql_do_connect(INTERNAL_FUNCTION_PARAMETERS,int persistent)
 				pgsql=PQsetdb(host,port,options,tty,dbname);
 			}
 			if (pgsql==NULL || PQstatus(pgsql)==CONNECTION_BAD) {
+				if (pgsql) {
+					PQfinish(pgsql);
+				}
 				php_error(E_WARNING,"Unable to connect to PostgreSQL server:  %s",PQerrorMessage(pgsql));
 				efree(hashed_details);
 				RETURN_FALSE;
 			}
 
+ 			PQsetNoticeProcessor(pgsql, _notice_handler, NULL);
+
 			/* hash it up */
 			new_le.type = le_plink;
 			new_le.ptr = pgsql;
@@ -372,6 +432,8 @@ void php_pgsql_do_connect(INTERNAL_FUNCTION_PARAMETERS,int persistent)
 				}
 			}
 			pgsql = (PGconn *) le->ptr;
+
+ 			PQsetNoticeProcessor(pgsql, _notice_handler, NULL);
 		}
 		ZEND_REGISTER_RESOURCE(return_value, pgsql, le_plink);
 	} else {
@@ -393,9 +455,9 @@ void php_pgsql_do_connect(INTERNAL_FUNCTION_PARAMETERS,int persistent)
 			ptr = zend_list_find(link,&type);   /* check if the link is still there */
 			if (ptr && (type==le_link || type==le_plink)) {
 				return_value->value.lval = link;
+				zend_list_addref(link);
 				php_pgsql_set_default_link(link);
 				return_value->type = IS_RESOURCE;
-				zend_list_addref(link);
 				efree(hashed_details);
 				return;
 			} else {
@@ -418,6 +480,8 @@ void php_pgsql_do_connect(INTERNAL_FUNCTION_PARAMETERS,int persistent)
 			RETURN_FALSE;
 		}
 
+		PQsetNoticeProcessor(pgsql, _notice_handler, NULL);
+
 		/* add it to the list */
 		ZEND_REGISTER_RESOURCE(return_value, pgsql, le_link);
 
@@ -819,6 +883,19 @@ PHP_FUNCTION(pg_cmdtuples)
 }
 /* }}} */
 
+/* {{{ proto int pg_last_notice(int connection)
+   Returns the last notice set by the backend */
+PHP_FUNCTION(pg_last_notice) 
+{
+	PGLS_FETCH();
+
+	if (PGG(last_notice) == NULL) {
+		RETURN_FALSE;
+	} else {       
+		RETURN_STRING(PGG(last_notice),0);
+	}
+}
+/* }}} */
 
 char *get_field_name(PGconn *pgsql, Oid oid, HashTable *list)
 {
@@ -1049,11 +1126,11 @@ static void php_pgsql_fetch_hash(INTERNAL_FUNCTION_PARAMETERS, int result_type)
 	for (i = 0, num_fields = PQnfields(pgsql_result); i<num_fields; i++) {
 		if (PQgetisnull(pgsql_result, Z_LVAL_PP(row), i)) {
 			if (result_type & PGSQL_NUM) {
-				add_index_unset(return_value, i);
+				add_index_null(return_value, i);
 			}
 			if (result_type & PGSQL_ASSOC) {
 				field_name = PQfname(pgsql_result, i);
-				add_assoc_unset(return_value, field_name);
+				add_assoc_null(return_value, field_name);
 			}
 		} else {
 			element = PQgetvalue(pgsql_result, Z_LVAL_PP(row), i);
