@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,10 +16,14 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: pspell.c,v 1.1.1.2 2001/01/25 04:59:54 wsanchez Exp $ */
+/* $Id: pspell.c,v 1.1.1.3 2001/07/19 00:19:54 zarzycki Exp $ */
 
 #define IS_EXT_MODULE
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "php.h"
 
 #include <stdlib.h>
@@ -28,6 +32,9 @@
 
 #if HAVE_PSPELL
 
+/* this will enforce compatibility in .12 version (broken after .11.2) */
+#define USE_ORIGINAL_MANAGER_FUNCS
+
 #include "php_pspell.h"
 #include <pspell/pspell.h>
 #include "ext/standard/info.h"
