@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: scanf.c,v 1.1.1.3 2001/01/25 05:00:12 wsanchez Exp $ */
+/* $Id: scanf.c,v 1.1.1.4 2001/07/19 00:20:21 zarzycki Exp $ */
 
 /*
    scanf.c --
@@ -663,7 +663,7 @@ PHPAPI int php_sscanf_internal(	char *string,char *format,
             return FAILURE;
         }
         for (i = 0; i < totalVars; i++) {
-           	if (add_next_index_unset(*return_value) == FAILURE) {
+           	if (add_next_index_null(*return_value) == FAILURE) {
 				scan_set_error_return(0, return_value);
 				return FAILURE;
 	   	}
