@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,13 +12,17 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Author: Sterling Hughes <Sterling.Hughes@pentap.net>                 |
+   | Author: Sterling Hughes <sterling@php.net>                           |
    +----------------------------------------------------------------------+
 */
 
-/* $Id: swf.c,v 1.1.1.3 2001/01/25 04:59:58 wsanchez Exp $ */
+/* $Id: swf.c,v 1.1.1.4 2001/07/19 00:20:25 zarzycki Exp $ */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "php.h"
 
 #if HAVE_SWF
@@ -190,7 +194,7 @@ PHP_FUNCTION(swf_openfile)
 			free_na = 0;
 			RETURN_FALSE;
 		}
-		V_UNLINK((const char *)na);
+		VCWD_UNLINK((const char *)na);
 		fclose(fp);
 		free_na = 1;
 		SWFG(use_file) = 0;
@@ -236,7 +240,7 @@ PHP_FUNCTION(swf_closefile)
 		char buf[4096];
 		int b;
 		
-		if ((f = V_FOPEN(SWFG(tmpfile_name), "r")) == NULL) {
+		if ((f = VCWD_FOPEN(SWFG(tmpfile_name), "r")) == NULL) {
 			php_error(E_WARNING, "Cannot create temporary file for stdout support with SWF");
 			RETURN_NULL();
 		}
@@ -246,7 +250,7 @@ PHP_FUNCTION(swf_closefile)
 		
 		fclose(f);
 		
-		V_UNLINK((const char *)SWFG(tmpfile_name));
+		VCWD_UNLINK((const char *)SWFG(tmpfile_name));
 	}
 }
 /* }}} */
