@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,11 +16,16 @@
    |          Stefan Rhrich <sr@linux.de>                                |
    +----------------------------------------------------------------------+
  */
-/* $Id: zlib.c,v 1.1.1.3 2001/01/25 05:00:22 wsanchez Exp $ */
+/* $Id: zlib.c,v 1.1.1.4 2001/07/19 00:20:34 zarzycki Exp $ */
 #define IS_EXT_MODULE
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "php.h"
 #include "SAPI.h"
+#include "php_ini.h"
 
 #include <stdlib.h>
 #include <errno.h>
@@ -41,7 +46,7 @@
 #include "ext/standard/php_standard.h"
 #include "ext/standard/info.h"
 #include "php_zlib.h"
-#include "fopen-wrappers.h"
+#include "fopen_wrappers.h"
 #if HAVE_PWD_H
 #ifdef PHP_WIN32
 #include "win32/pwd.h"
@@ -106,12 +111,18 @@ function_entry php_zlib_functions[] = {
 	{NULL, NULL, NULL}
 };
 
+
+PHP_INI_BEGIN()
+    STD_PHP_INI_BOOLEAN("zlib.output_compression",   "0",    PHP_INI_ALL,     OnUpdateInt,        output_compression,   php_zlib_globals,     zlib_globals)
+PHP_INI_END()
+
+
 zend_module_entry php_zlib_module_entry = {
 	"zlib",
 	php_zlib_functions,
 	PHP_MINIT(zlib),
 	PHP_MSHUTDOWN(zlib),
-	NULL,
+	PHP_RINIT(zlib),
 	NULL,
 	PHP_MINFO(zlib),
 	STANDARD_MODULE_PROPERTIES
@@ -155,9 +166,29 @@ PHP_MINIT_FUNCTION(zlib)
 	REGISTER_LONG_CONSTANT("FORCE_GZIP", CODING_GZIP, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("FORCE_DEFLATE", CODING_DEFLATE, CONST_CS | CONST_PERSISTENT);
 
+	REGISTER_INI_ENTRIES();
+
 	return SUCCESS;
 }
 
+PHP_RINIT_FUNCTION(zlib)
+{
+	ZLIBLS_FETCH();
+
+	ZLIBG(ob_gzhandler_status) = 0;
+	switch (ZLIBG(output_compression)) {
+		case 0:
+			break;
+		case 1:
+			php_enable_output_compression(4096);
+			break;
+		default:
+			php_enable_output_compression(ZLIBG(output_compression));
+	}
+	return SUCCESS;
+}
+
+
 PHP_MSHUTDOWN_FUNCTION(zlib)
 {
 #if HAVE_FOPENCOOKIE
@@ -168,6 +199,8 @@ PHP_MSHUTDOWN_FUNCTION(zlib)
     }
 #endif
 	
+	UNREGISTER_INI_ENTRIES();
+
 	return SUCCESS;
 }
 
@@ -930,108 +963,120 @@ PHP_FUNCTION(gzinflate)
 
 
 
-static int php_do_deflate(uint str_length, Bytef **p_buffer, uint *p_buf_used ZLIBLS_DC)
+static int php_do_deflate(uint str_length, Bytef **p_buffer, uint *p_buffer_len, zend_bool do_start, zend_bool do_end ZLIBLS_DC)
 {
 	Bytef *buffer;
 	uInt prev_outlen, outlen;
 	int err;
+	int start_offset = (do_start?10:0);
+	int end_offset = (do_end?8:0);
 
 	outlen = sizeof(char) * (str_length * 1.001 + 12);
-	buffer = (Bytef *) emalloc(outlen+10+8); /* 10+8 for the header and trailer */
+	if ((outlen+start_offset+end_offset) > *p_buffer_len) {
+		buffer = (Bytef *) emalloc(outlen+start_offset+end_offset);
+	} else {
+		buffer = *p_buffer;
+	}
 	
-	ZLIBG(stream).next_out = buffer+10;
+	ZLIBG(stream).next_out = buffer+start_offset;
 	ZLIBG(stream).avail_out = outlen;
 
-	err = deflate(&ZLIBG(stream), Z_NO_FLUSH);
+
+	err = deflate(&ZLIBG(stream), Z_SYNC_FLUSH);
 	while (err == Z_OK && !ZLIBG(stream).avail_out) {
 		prev_outlen = outlen;
 		outlen *= 3;
-		buffer = realloc(buffer, outlen+10+8);
+		if ((outlen+start_offset+end_offset) > *p_buffer_len) {
+			buffer = realloc(buffer, outlen+start_offset+end_offset);
+		}
 		
-		ZLIBG(stream).next_out = buffer+10 + prev_outlen;
+		ZLIBG(stream).next_out = buffer+start_offset + prev_outlen;
 		ZLIBG(stream).avail_out = prev_outlen * 2;
 
-		err = deflate(&ZLIBG(stream), Z_NO_FLUSH);
+		err = deflate(&ZLIBG(stream), Z_SYNC_FLUSH);
+	}
+
+	if (do_end) {
+		err = deflate(&ZLIBG(stream), Z_FINISH);
 	}
 
-	err = deflate(&ZLIBG(stream), Z_FINISH);
 
 	*p_buffer = buffer;
-	*p_buf_used = outlen - ZLIBG(stream).avail_out;
+	*p_buffer_len = outlen - ZLIBG(stream).avail_out;
+
 	return err;
 }
 
 
-int php_deflate_string(const char *str, uint str_length, char **newstr, uint *new_length, int coding)
+int php_deflate_string(const char *str, uint str_length, char **newstr, uint *new_length, int coding, zend_bool do_start, zend_bool do_end)
 {
-	Bytef *buffer;
-	uInt buf_used;
 	int err;
-	Bytef header_buffer[11];
-	Bytef trailer_buffer[9];
 	ZLIBLS_FETCH();
 
 	ZLIBG(compression_coding) = coding;
 
-	ZLIBG(stream).zalloc = Z_NULL;
-	ZLIBG(stream).zfree = Z_NULL;
-	ZLIBG(stream).opaque = Z_NULL;
-
-	switch (coding) {
-		case CODING_GZIP:
-			/* windowBits is passed < 0 to suppress zlib header & trailer */
-			if (deflateInit2(&ZLIBG(stream), Z_DEFAULT_COMPRESSION, Z_DEFLATED,
-					-MAX_WBITS, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY)
-						!= Z_OK) {
-				/* TODO: print out error */
-				return FAILURE;
-			}
+	if (do_start) {
+		ZLIBG(stream).zalloc = Z_NULL;
+		ZLIBG(stream).zfree = Z_NULL;
+		ZLIBG(stream).opaque = Z_NULL;
+		switch (coding) {
+			case CODING_GZIP:
+				/* windowBits is passed < 0 to suppress zlib header & trailer */
+				if (deflateInit2(&ZLIBG(stream), Z_DEFAULT_COMPRESSION, Z_DEFLATED,
+						-MAX_WBITS, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY)
+							!= Z_OK) {
+					/* TODO: print out error */
+					return FAILURE;
+				}
 		
-			/* Write a very simple .gz header: */
-			sprintf(header_buffer, "%c%c%c%c%c%c%c%c%c%c", gz_magic[0],
-						gz_magic[1], Z_DEFLATED, 0 /*flags*/,
-						0,0,0,0 /*time*/, 0 /*xflags*/, OS_CODE);
-			ZLIBG(crc) = crc32(0L, Z_NULL, 0);
-			break;
-		case CODING_DEFLATE:
-			if (deflateInit(&ZLIBG(stream), Z_DEFAULT_COMPRESSION) != Z_OK) {
-				/* TODO: print out error */
-				return FAILURE;
-			}
-			break;		
+				ZLIBG(crc) = crc32(0L, Z_NULL, 0);
+				break;
+			case CODING_DEFLATE:
+				if (deflateInit(&ZLIBG(stream), Z_DEFAULT_COMPRESSION) != Z_OK) {
+					/* TODO: print out error */
+					return FAILURE;
+				}
+				break;		
+		}
 	}
 
 
 	ZLIBG(stream).next_in = (Bytef*) str;
 	ZLIBG(stream).avail_in = (uInt) str_length;
-	
+
 	if (ZLIBG(compression_coding) == 1) {
-		ZLIBG(crc) = crc32(ZLIBG(crc), (const Bytef *)str, str_length);
+		ZLIBG(crc) = crc32(ZLIBG(crc), (const Bytef *) str, str_length);
 	}
 
-	err = php_do_deflate(str_length, &buffer, &buf_used ZLIBLS_CC);
+	err = php_do_deflate(str_length, (Bytef **) newstr, new_length, do_start, do_end ZLIBLS_CC);
 	/* TODO: error handling (err may be Z_STREAM_ERROR, Z_BUF_ERROR, ?) */
 
-
-	if (ZLIBG(compression_coding) == 1) {
-		/* write crc & stream.total_in in LSB order */
-		sprintf(trailer_buffer, "%c%c%c%c%c%c%c%c",
-					(char) ZLIBG(crc) & 0xFF,
-					(char) (ZLIBG(crc) >> 8) & 0xFF,
-					(char) (ZLIBG(crc) >> 16) & 0xFF,
-					(char) (ZLIBG(crc) >> 24) & 0xFF,
-					(char) ZLIBG(stream).total_in & 0xFF,
-					(char) (ZLIBG(stream).total_in >> 8) & 0xFF,
-					(char) (ZLIBG(stream).total_in >> 16) & 0xFF,
-					(char) (ZLIBG(stream).total_in >> 24) & 0xFF);
+	if (do_start) {
+		/* Write a very simple .gz header: */
+		(*newstr)[0] = gz_magic[0];
+		(*newstr)[1] = gz_magic[1];
+		(*newstr)[2] = Z_DEFLATED;
+		(*newstr)[3] = (*newstr)[4] = (*newstr)[5] = (*newstr)[6] = (*newstr)[7] = (*newstr)[8] = 0;
+		(*newstr)[9] = OS_CODE;
+		*new_length += 10;
 	}
-	
+	if (do_end) {
+		if (ZLIBG(compression_coding) == 1) {
+			char *trailer = (*newstr)+(*new_length);
 
-	memcpy(buffer, header_buffer, 10);
-	memcpy(buffer+10+buf_used, trailer_buffer, 8);
-
-	*new_length = buf_used + 10 + 8;
-	*newstr = buffer;
+			/* write crc & stream.total_in in LSB order */
+			trailer[0] = (char) ZLIBG(crc) & 0xFF;
+			trailer[1] = (char) (ZLIBG(crc) >> 8) & 0xFF;
+			trailer[2] = (char) (ZLIBG(crc) >> 16) & 0xFF;
+			trailer[3] = (char) (ZLIBG(crc) >> 24) & 0xFF;
+			trailer[4] = (char) ZLIBG(stream).total_in & 0xFF;
+			trailer[5] = (char) (ZLIBG(stream).total_in >> 8) & 0xFF;
+			trailer[6] = (char) (ZLIBG(stream).total_in >> 16) & 0xFF;
+			trailer[7] = (char) (ZLIBG(stream).total_in >> 24) & 0xFF;
+			*new_length += 8;
+		}
+		deflateEnd(&ZLIBG(stream));
+	}
 
 	return SUCCESS;
 }
@@ -1043,14 +1088,14 @@ PHP_FUNCTION(gzencode)
 	int coding;
 
 	switch(ZEND_NUM_ARGS()) {
-		case CODING_GZIP:
+		case 1:
 			if (zend_get_parameters_ex(1, &zv_string)==FAILURE) {
 				RETURN_FALSE;
 			}
 			convert_to_string_ex(zv_string);
 			coding = 1;
 			break;
-		case CODING_DEFLATE:
+		case 2:
 			if (zend_get_parameters_ex(2, &zv_string, &zv_coding)==FAILURE) {
 				RETURN_FALSE;
 			}
@@ -1062,7 +1107,7 @@ PHP_FUNCTION(gzencode)
 			ZEND_WRONG_PARAM_COUNT();
 			break;
 	}
-	if (php_deflate_string(Z_STRVAL_PP(zv_string), Z_STRLEN_PP(zv_string), &Z_STRVAL_P(return_value), &Z_STRLEN_P(return_value), coding)==SUCCESS) {
+	if (php_deflate_string(Z_STRVAL_PP(zv_string), Z_STRLEN_PP(zv_string), &Z_STRVAL_P(return_value), &Z_STRLEN_P(return_value), coding, 1, 1)==SUCCESS) {
 		Z_TYPE_P(return_value) = IS_STRING;
 	} else {
 		RETURN_FALSE;
@@ -1073,20 +1118,24 @@ PHP_FUNCTION(gzencode)
 PHP_FUNCTION(ob_gzhandler)
 {
 	int coding;
-	zval **zv_string;
+	zval **zv_string, **zv_mode;
 	zval **data, **a_encoding;
 	zend_bool return_original=0;
+	zend_bool do_start, do_end;
+	ZLIBLS_FETCH();
 
-	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &zv_string)==FAILURE) {
+	if (ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2, &zv_string, &zv_mode)==FAILURE) {
 		ZEND_WRONG_PARAM_COUNT();
 	}
 
-	if (zend_hash_find(&EG(symbol_table), "HTTP_SERVER_VARS", sizeof("HTTP_SERVER_VARS"), (void **) &data)==FAILURE
+	if (ZLIBG(ob_gzhandler_status)==-1
+		|| zend_hash_find(&EG(symbol_table), "HTTP_SERVER_VARS", sizeof("HTTP_SERVER_VARS"), (void **) &data)==FAILURE
 		|| Z_TYPE_PP(data)!=IS_ARRAY
 		|| zend_hash_find(Z_ARRVAL_PP(data), "HTTP_ACCEPT_ENCODING", sizeof("HTTP_ACCEPT_ENCODING"), (void **) &a_encoding)==FAILURE) {
 		/* return the original string */
 		*return_value = **zv_string;
 		zval_copy_ctor(return_value);
+		ZLIBG(ob_gzhandler_status)=-1;
 		return;
 	}
 	convert_to_string_ex(a_encoding);
@@ -1097,25 +1146,47 @@ PHP_FUNCTION(ob_gzhandler)
 	} else {
 		RETURN_FALSE;
 	}
-	if (php_deflate_string(Z_STRVAL_PP(zv_string), Z_STRLEN_PP(zv_string), &Z_STRVAL_P(return_value), &Z_STRLEN_P(return_value), coding)==SUCCESS) {
+	
+	convert_to_long_ex(zv_mode);
+	do_start = ((Z_LVAL_PP(zv_mode) & PHP_OUTPUT_HANDLER_START) ? 1 : 0);
+	do_end = ((Z_LVAL_PP(zv_mode) & PHP_OUTPUT_HANDLER_END) ? 1 : 0);
+	Z_STRVAL_P(return_value) = NULL;
+	Z_STRLEN_P(return_value) = 0;
+	if (php_deflate_string(Z_STRVAL_PP(zv_string), Z_STRLEN_PP(zv_string), &Z_STRVAL_P(return_value), &Z_STRLEN_P(return_value), coding, do_start, do_end)==SUCCESS) {
 		Z_TYPE_P(return_value) = IS_STRING;
-		switch (coding) {
-			case CODING_GZIP:
-				if (sapi_add_header("Content-Encoding: gzip", sizeof("Content-Encoding: gzip") - 1, 1)==FAILURE) {
+		if (do_start) {
+			switch (coding) {
+				case CODING_GZIP:
+					if (sapi_add_header("Content-Encoding: gzip", sizeof("Content-Encoding: gzip") - 1, 1)==FAILURE) {
+						return_original = 1;
+					}
+					if (sapi_add_header("Vary: Accept-Encoding", sizeof("Vary: Accept-Encoding") - 1, 1)==FAILURE) {
+						return_original = 1;
+					}
+					break;
+				case CODING_DEFLATE:
+					if (sapi_add_header("Content-Encoding: deflate", sizeof("Content-Encoding: deflate") - 1, 1)==FAILURE) {
+						return_original = 1;
+					}
+					if (sapi_add_header("Vary: Accept-Encoding", sizeof("Vary: Accept-Encoding") - 1, 1)==FAILURE) {
+						return_original = 1;
+					}
+					break;
+				default:
 					return_original = 1;
-				}
-				break;
-			case CODING_DEFLATE:
-				if (sapi_add_header("Content-Encoding: deflate", sizeof("Content-Encoding: deflate") - 1, 1)==FAILURE) {
-					return_original = 1;
-				}
-				break;
-			default:
-				return_original = 1;
-				break;
+					break;
+			}
 		}
+
 		if (return_original) {
 			zval_dtor(return_value);
+#if 0
+		} else {
+			char lenbuf[64];
+			
+			sprintf(lenbuf,"Content-Length: %d",Z_STRLEN_P(return_value));
+			sapi_add_header(lenbuf,strlen(lenbuf), 1);
+#endif
 		}
 	} else {
 		return_original = 1;
@@ -1129,9 +1200,47 @@ PHP_FUNCTION(ob_gzhandler)
 }
 
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- */
+
+static void php_gzip_output_handler(char *output, uint output_len, char **handled_output, uint *handled_output_len, int mode)
+{
+	zend_bool do_start, do_end;
+	ZLIBLS_FETCH();
+
+	do_start = (mode & PHP_OUTPUT_HANDLER_START ? 1 : 0);
+	do_end = (mode & PHP_OUTPUT_HANDLER_END ? 1 : 0);
+	if (php_deflate_string(output, output_len, handled_output, handled_output_len, ZLIBG(ob_gzip_coding), do_start, do_end)!=SUCCESS) {
+		zend_error(E_ERROR, "Compression failed");
+	}
+}
+
+
+int php_enable_output_compression(int buffer_size)
+{
+	zval **a_encoding, **data;
+	ELS_FETCH();
+	ZLIBLS_FETCH();
+
+	if (zend_hash_find(&EG(symbol_table), "HTTP_SERVER_VARS", sizeof("HTTP_SERVER_VARS"), (void **) &data)==FAILURE
+		|| Z_TYPE_PP(data)!=IS_ARRAY
+		|| zend_hash_find(Z_ARRVAL_PP(data), "HTTP_ACCEPT_ENCODING", sizeof("HTTP_ACCEPT_ENCODING"), (void **) &a_encoding)==FAILURE) {
+		return FAILURE;
+	}
+	convert_to_string_ex(a_encoding);
+	if (php_memnstr(Z_STRVAL_PP(a_encoding), "gzip", 4, Z_STRVAL_PP(a_encoding) + Z_STRLEN_PP(a_encoding))) {
+		if (sapi_add_header("Content-Encoding: gzip", sizeof("Content-Encoding: gzip") - 1, 1)==FAILURE) {
+			return FAILURE;
+		}
+		ZLIBG(ob_gzip_coding) = CODING_GZIP;
+	} else if(php_memnstr(Z_STRVAL_PP(a_encoding), "deflate", 7, Z_STRVAL_PP(a_encoding) + Z_STRLEN_PP(a_encoding))) {
+		if (sapi_add_header("Content-Encoding: deflate", sizeof("Content-Encoding: deflate") - 1, 1)==FAILURE) {
+			return FAILURE;
+		}
+		ZLIBG(ob_gzip_coding) = CODING_DEFLATE;
+	} else {
+		return FAILURE;
+	}
+	
+	php_start_ob_buffer(NULL, buffer_size);
+	php_ob_set_internal_handler(php_gzip_output_handler, buffer_size*1.5);
+	return SUCCESS;
+}
