@@ -3,7 +3,7 @@
 // +----------------------------------------------------------------------+
 // | PHP version 4.0                                                      |
 // +----------------------------------------------------------------------+
-// | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+// | Copyright (c) 1997-2001 The PHP Group                                |
 // +----------------------------------------------------------------------+
 // | This source file is subject to version 2.0 of the PHP license,       |
 // | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
 // |          Urs Gehrig <urs@circle.ch>                                  |
 // +----------------------------------------------------------------------+
 //
-// $Id: Form.php,v 1.1.1.2 2001/01/25 05:00:30 wsanchez Exp $
+// $Id: Form.php,v 1.1.1.3 2001/07/19 00:20:49 zarzycki Exp $
 //
 // HTML form utility functions.
 //
@@ -282,8 +282,8 @@ class HTML_Form {
     // }}}
     // {{{ displayResetRow()
 
-    function displayReset($title = 'Clear contents') {
-        print $this->returnReset($title);
+    function displayResetRow($title = 'Clear contents') {
+        print $this->returnResetRow($title);
     }
 
     // }}}
